"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.sendTail = exports.openTailStream = void 0;
const sp_common_1 = require("@streamdal/protos/protos/sp_common");
let tailStream = null;
const openTailStream = (configs) => {
    try {
        return configs.grpcClient.sendTail({
            meta: { "auth-token": configs.streamdalToken },
        });
    }
    catch (e) {
        console.error("error opening tail client stream");
        return null;
    }
};
exports.openTailStream = openTailStream;
const sendTail = ({ configs, tails, audience, originalData, newData, }) => {
    // eslint-disable-next-line @typescript-eslint/no-misused-promises
    tails === null || tails === void 0 ? void 0 : tails.forEach((tailStatus, tailRequestId) => __awaiter(void 0, void 0, void 0, function* () {
        if (!tailStatus.tail) {
            return;
        }
        try {
            if (tailStream == null) {
                console.info("tail stream is closed, reopening...");
                tailStream = (0, exports.openTailStream)(configs);
            }
            const tailResponse = sp_common_1.TailResponse.create({
                timestampNs: (BigInt(new Date().getTime()) * BigInt(1e6)).toString(),
                type: sp_common_1.TailResponseType.PAYLOAD,
                tailRequestId: tailRequestId,
                audience,
                sessionId: configs.sessionId,
                originalData,
                newData,
            });
            console.debug("sending tail response", tailResponse);
            yield (tailStream === null || tailStream === void 0 ? void 0 : tailStream.requests.send(tailResponse));
            const headers = yield (tailStream === null || tailStream === void 0 ? void 0 : tailStream.headers);
            console.debug("got tail response headers: ", headers);
            const response = yield (tailStream === null || tailStream === void 0 ? void 0 : tailStream.response);
            console.debug("got tail response message: ", response);
            const status = yield (tailStream === null || tailStream === void 0 ? void 0 : tailStream.status);
            console.debug("got tail status: ", status);
            const trailers = yield (tailStream === null || tailStream === void 0 ? void 0 : tailStream.trailers);
            console.debug("got tail trailers: ", trailers);
        }
        catch (e) {
            tailStream = null;
            console.error("Error sending tail", e);
        }
    }));
};
exports.sendTail = sendTail;
